@@ -23,6 +23,8 @@
 #ifndef DEVICE_H_
 #define DEVICE_H_
 
+#define MAX_LENGTH_OF_DEVICE_STRING 8
+
 void nwipe_device_identify( nwipe_context_t* c );  // Get hardware information about the device.
 int nwipe_device_scan( nwipe_context_t*** c );  // Find devices that we can wipe.
 int nwipe_device_get( nwipe_context_t*** c, char** devnamelist, int ndevnames );  // Get info about devices to wipe.
@@ -30,5 +32,6 @@ int nwipe_get_device_bus_type_and_serialno( char*, nwipe_device_t*, char* );
 void strip_CR_LF( char* );
 void determine_disk_capacity_nomenclature( u64, char* );
 void remove_ATA_prefix( char* );
+char* trim( char* );
 
 #endif /* DEVICE_H_ */
