@@ -51,6 +51,9 @@ void nwipe_log( nwipe_log_t level, const char* format, ... )
      *
      */
 
+    extern int terminate_signal;
+    extern int user_abort;
+
     char** result;
     char* malloc_result;
     char message_buffer[MAX_LOG_LINE_CHARS * sizeof( char )];
@@ -326,57 +329,64 @@ void nwipe_log( nwipe_log_t level, const char* format, ... )
         /* Open the log file for appending. */
         fp = fopen( nwipe_options.logfile, "a" );
 
-        if( fp == NULL )
+        if( fp != NULL )
         {
-            fprintf( stderr, "nwipe_log: Unable to open '%s' for logging.\n", nwipe_options.logfile );
+
+            /* Get the file descriptor of the log file. */
+            fd = fileno( fp );
+
+            /* Block and lock. */
+            r = flock( fd, LOCK_EX );
+
+            if( r != 0 )
+            {
+                perror( "nwipe_log: flock:" );
+                fprintf( stderr, "nwipe_log: Unable to lock '%s' for logging.\n", nwipe_options.logfile );
+                r = pthread_mutex_unlock( &mutex1 );
+                if( r != 0 )
+                {
+                    fprintf( stderr, "nwipe_log: pthread_mutex_unlock failed. Code %i \n", r );
+
+                    /* Unlock the file. */
+                    r = flock( fd, LOCK_UN );
+                    fclose( fp );
+                    return;
+                }
+            }
+
+            fprintf( fp, "%s\n", log_lines[log_current_element] );
+
+            /* Unlock the file. */
+            r = flock( fd, LOCK_UN );
+
+            if( r != 0 )
+            {
+                perror( "nwipe_log: flock:" );
+                fprintf( stderr, "Error: Unable to unlock '%s' after logging.\n", nwipe_options.logfile );
+            }
+
+            /* Close the stream. */
+            r = fclose( fp );
+
+            if( r != 0 )
+            {
+                perror( "nwipe_log: fclose:" );
+                fprintf( stderr, "Error: Unable to close '%s' after logging.\n", nwipe_options.logfile );
+            }
+        }
+        else
+        {
+            /* Tell user we can't create/open the log and terminate nwipe */
+            fprintf(
+                stderr, "\nERROR:Unable to create/open '%s' for logging, permissions?\n\n", nwipe_options.logfile );
             r = pthread_mutex_unlock( &mutex1 );
             if( r != 0 )
             {
                 fprintf( stderr, "nwipe_log: pthread_mutex_unlock failed. Code %i \n", r );
-                return;
             }
-        }
-
-        /* Get the file descriptor of the log file. */
-        fd = fileno( fp );
-
-        /* Block and lock. */
-        r = flock( fd, LOCK_EX );
-
-        if( r != 0 )
-        {
-            perror( "nwipe_log: flock:" );
-            fprintf( stderr, "nwipe_log: Unable to lock '%s' for logging.\n", nwipe_options.logfile );
-            r = pthread_mutex_unlock( &mutex1 );
-            if( r != 0 )
-            {
-                fprintf( stderr, "nwipe_log: pthread_mutex_unlock failed. Code %i \n", r );
-
-                /* Unlock the file. */
-                r = flock( fd, LOCK_UN );
-                fclose( fp );
-                return;
-            }
-        }
-
-        fprintf( fp, "%s\n", log_lines[log_current_element] );
-
-        /* Unlock the file. */
-        r = flock( fd, LOCK_UN );
-
-        if( r != 0 )
-        {
-            perror( "nwipe_log: flock:" );
-            fprintf( stderr, "Error: Unable to unlock '%s' after logging.\n", nwipe_options.logfile );
-        }
-
-        /* Close the stream. */
-        r = fclose( fp );
-
-        if( r != 0 )
-        {
-            perror( "nwipe_log: fclose:" );
-            fprintf( stderr, "Error: Unable to close '%s' after logging.\n", nwipe_options.logfile );
+            user_abort = 1;
+            terminate_signal = 1;
+            return;
         }
     }
 
@@ -498,29 +508,36 @@ int nwipe_log_sysinfo()
      * be logged, making sure the last entry in the array is a NULL string. To remove
      * an entry simply comment out the keyword with //
      */
-    char dmidecode_keywords[][24] = {
-        "bios-version",
-        "bios-release-date",
-        "system-manufacturer",
-        "system-product-name",
-        "system-version",
-        "system-serial-number",
-        "system-uuid",
-        "baseboard-manufacturer",
-        "baseboard-product-name",
-        "baseboard-version",
-        "baseboard-serial-number",
-        "baseboard-asset-tag",
-        "chassis-manufacturer",
-        "chassis-type",
-        "chassis-version",
-        "chassis-serial-number",
-        "chassis-asset-tag",
-        "processor-family",
-        "processor-manufacturer",
-        "processor-version",
-        "processor-frequency",
-        ""  // terminates the keyword array. DO NOT REMOVE
+
+    /* The 0/1 after the keyword determines whether the data for this
+     * keyword is displayed when -q (anonymize) has been specified
+     * by the user. An quick reminder about multi dimensional arrays, the first
+     * []=the keyword (0-21) including the empty string. The second [] is the
+     * 1 or 0 value (0 or 1). The third [] is the index value into either string.
+     */
+    char dmidecode_keywords[][2][24] = {
+        { "bios-version", "1" },
+        { "bios-release-date", "1" },
+        { "system-manufacturer", "1" },
+        { "system-product-name", "1" },
+        { "system-version", "1" },
+        { "system-serial-number", "0" },
+        { "system-uuid", "0" },
+        { "baseboard-manufacturer", "1" },
+        { "baseboard-product-name", "1" },
+        { "baseboard-version", "1" },
+        { "baseboard-serial-number", "0" },
+        { "baseboard-asset-tag", "0" },
+        { "chassis-manufacturer", "1" },
+        { "chassis-type", "1" },
+        { "chassis-version", "1" },
+        { "chassis-serial-number", "0" },
+        { "chassis-asset-tag", "0" },
+        { "processor-family", "1" },
+        { "processor-manufacturer", "1" },
+        { "processor-version", "1" },
+        { "processor-frequency", "1" },
+        { "", "" }  // terminates the keyword array. DO NOT REMOVE
     };
 
     char dmidecode_command[] = "dmidecode -s %s";
@@ -563,9 +580,9 @@ int nwipe_log_sysinfo()
     {
 
         /* Run the dmidecode command to retrieve each dmidecode keyword, one at a time */
-        while( dmidecode_keywords[keywords_idx][0] != 0 )
+        while( dmidecode_keywords[keywords_idx][0][0] != 0 )
         {
-            sprintf( cmd, p_dmidecode_command, &dmidecode_keywords[keywords_idx][0] );
+            sprintf( cmd, p_dmidecode_command, &dmidecode_keywords[keywords_idx][0][0] );
             fp = popen( cmd, "r" );
             if( fp == NULL )
             {
@@ -581,7 +598,22 @@ int nwipe_log_sysinfo()
                 {
                     path[len - 1] = 0;
                 }
-                nwipe_log( NWIPE_LOG_NOTICE, "%s = %s", &dmidecode_keywords[keywords_idx][0], path );
+                if( nwipe_options.quiet )
+                {
+                    if( *( &dmidecode_keywords[keywords_idx][1][0] ) == '0' )
+                    {
+                        nwipe_log(
+                            NWIPE_LOG_NOTICE, "%s = %s", &dmidecode_keywords[keywords_idx][0][0], "XXXXXXXXXXXXXXX" );
+                    }
+                    else
+                    {
+                        nwipe_log( NWIPE_LOG_NOTICE, "%s = %s", &dmidecode_keywords[keywords_idx][0][0], path );
+                    }
+                }
+                else
+                {
+                    nwipe_log( NWIPE_LOG_NOTICE, "%s = %s", &dmidecode_keywords[keywords_idx][0][0], path );
+                }
             }
             /* close */
             r = pclose( fp );
@@ -607,7 +639,7 @@ void nwipe_log_summary( nwipe_context_t** ptr, int nwipe_selected )
     int i;
     int idx_src;
     int idx_dest;
-    char device[7];
+    char device[18];
     char status[9];
     char throughput[13];
     char total_throughput_string[13];
@@ -662,24 +694,21 @@ void nwipe_log_summary( nwipe_context_t** ptr, int nwipe_selected )
     nwipe_log( NWIPE_LOG_NOTIMESTAMP, "" );
     nwipe_log( NWIPE_LOG_NOTIMESTAMP,
                "******************************** Error Summary *********************************" );
-    nwipe_log( NWIPE_LOG_NOTIMESTAMP, "! Device | Pass Errors | Verifications Errors |" );
+    nwipe_log( NWIPE_LOG_NOTIMESTAMP, "!   Device | Pass Errors | Verifications Errors | Fdatasync I\\O Errors" );
     nwipe_log( NWIPE_LOG_NOTIMESTAMP,
                "--------------------------------------------------------------------------------" );
 
     for( i = 0; i < nwipe_selected; i++ )
     {
-        if( c[i]->pass_errors != 0 || c[i]->verify_errors != 0 )
+        if( c[i]->pass_errors != 0 || c[i]->verify_errors != 0 || c[i]->fsyncdata_errors != 0 )
         {
             strncpy( exclamation_flag, "!", 1 );
             exclamation_flag[1] = 0;
         }
         else
         {
-            if( c[i]->wipe_status == 0 )
-            {
-                strncpy( exclamation_flag, " ", 1 );
-                exclamation_flag[1] = 0;
-            }
+            strncpy( exclamation_flag, " ", 1 );
+            exclamation_flag[1] = 0;
         }
 
         /* Device name, strip any prefixed /dev/.. leaving up to 6 right justified
@@ -691,25 +720,15 @@ void nwipe_log_summary( nwipe_context_t** ptr, int nwipe_selected )
         idx_src = strlen( c[i]->device_name );
         idx_src--;
 
-        while( idx_dest >= 0 )
-        {
-            /* if the device name contains a / start prefixing spaces */
-            if( c[i]->device_name[idx_src] == '/' )
-            {
-                device[idx_dest--] = ' ';
-                continue;
-            }
-            if( idx_src >= 0 )
-            {
-                device[idx_dest--] = c[i]->device_name[idx_src--];
-            }
-        }
+        nwipe_strip_path( device, c[i]->device_name );
+
         nwipe_log( NWIPE_LOG_NOTIMESTAMP,
-                   "%s %s |  %10llu |           %10llu |",
+                   "%s %s |  %10llu |           %10llu |           %10llu",
                    exclamation_flag,
                    device,
                    c[i]->pass_errors,
-                   c[i]->verify_errors );
+                   c[i]->verify_errors,
+                   c[i]->fsyncdata_errors );
     }
 
     nwipe_log( NWIPE_LOG_NOTIMESTAMP,
@@ -728,44 +747,28 @@ void nwipe_log_summary( nwipe_context_t** ptr, int nwipe_selected )
     nwipe_log( NWIPE_LOG_NOTIMESTAMP, "" );
     nwipe_log( NWIPE_LOG_NOTIMESTAMP,
                "********************************* Drive Status *********************************" );
-    nwipe_log( NWIPE_LOG_NOTIMESTAMP, "! Device | Status | Thru-put | HH:MM:SS | Model/Serial Number" );
+    nwipe_log( NWIPE_LOG_NOTIMESTAMP, "!   Device | Status | Thru-put | HH:MM:SS | Model/Serial Number" );
     nwipe_log( NWIPE_LOG_NOTIMESTAMP,
                "--------------------------------------------------------------------------------" );
     /* Example layout:
-     *                                "!    sdv |--FAIL--|  120MB/s | 01:22:01 | WD6788.8488YNHj/ZX677888388-N       "
-     * ); "     sdv | Erased |  120MB/s | 01:25:04 | WD6784.8488JKGG/ZX677888388-N       " ); "     sdv | Erased |
+     *    "!     sdb |--FAIL--|  120MB/s | 01:22:01 | WD6788.8488YNHj/ZX677888388-N       "
+     * ); "      sdc | Erased |  120MB/s | 01:25:04 | WD6784.8488JKGG/ZX677888388-N       " ); "     sdv | Erased |
      * 120MB/s | 01:19:07 | WD6788.848HHDDR/ZX677888388-N       " ); End of Example layout */
 
     for( i = 0; i < nwipe_selected; i++ )
     {
-        /* Device name, strip any prefixed /dev/.. leaving up to 6 right justified
-         * characters eg "   sda", prefixed with space to 6 characters, note that
+        /* Device name, strip any prefixed /dev/.. leaving up to 8 right justified
+         * characters eg "   sda", prefixed with space to 8 characters, note that
          * we are processing the strings right to left */
 
-        idx_dest = 6;
-        device[idx_dest--] = 0;
-        idx_src = strlen( c[i]->device_name );
-        idx_src--;
+        nwipe_strip_path( device, c[i]->device_name );
 
-        while( idx_dest >= 0 )
-        {
-            /* if the device name contains a / start prefixing spaces */
-            if( c[i]->device_name[idx_src] == '/' )
-            {
-                device[idx_dest--] = ' ';
-                continue;
-            }
-            if( idx_src >= 0 )
-            {
-                device[idx_dest--] = c[i]->device_name[idx_src--];
-            }
-        }
         extern int user_abort;
 
         /* Any errors ? if so set the exclamation_flag and fail message,
          * All status messages should be eight characters EXACTLY !
          */
-        if( c[i]->result < 0 )
+        if( c[i]->pass_errors != 0 || c[i]->verify_errors != 0 || c[i]->fsyncdata_errors != 0 )
         {
             strncpy( exclamation_flag, "!", 1 );
             exclamation_flag[1] = 0;
@@ -775,44 +778,32 @@ void nwipe_log_summary( nwipe_context_t** ptr, int nwipe_selected )
         }
         else
         {
-
-            if( c[i]->pass_errors != 0 || c[i]->verify_errors != 0 )
+            if( c[i]->wipe_status == 0 )
             {
-                strncpy( exclamation_flag, "!", 1 );
+                strncpy( exclamation_flag, " ", 1 );
                 exclamation_flag[1] = 0;
 
-                strncpy( status, "-FAILED-", 8 );
+                strncpy( status, " Erased ", 8 );
                 status[8] = 0;
             }
             else
             {
-                if( c[i]->wipe_status == 0 )
+                if( user_abort == 1 )
                 {
-                    strncpy( exclamation_flag, " ", 1 );
+                    strncpy( exclamation_flag, "!", 1 );
                     exclamation_flag[1] = 0;
 
-                    strncpy( status, " Erased ", 8 );
+                    strncpy( status, "UABORTED", 8 );
                     status[8] = 0;
                 }
                 else
                 {
-                    if( user_abort == 1 )
-                    {
-                        strncpy( exclamation_flag, "!", 1 );
-                        exclamation_flag[1] = 0;
+                    /* If this ever happens, there is a bug ! */
+                    strncpy( exclamation_flag, " ", 1 );
+                    exclamation_flag[1] = 0;
 
-                        strncpy( status, "UABORTED", 8 );
-                        status[8] = 0;
-                    }
-                    else
-                    {
-                        /* If this ever happens, there is a bug ! */
-                        strncpy( exclamation_flag, " ", 1 );
-                        exclamation_flag[1] = 0;
-
-                        strncpy( status, "INSANITY", 8 );
-                        status[8] = 0;
-                    }
+                    strncpy( status, "INSANITY", 8 );
+                    status[8] = 0;
                 }
             }
         }
@@ -868,7 +859,7 @@ void nwipe_log_summary( nwipe_context_t** ptr, int nwipe_selected )
     /* Determine the size of throughput so that the correct nomenclature can be used */
     Determine_C_B_nomenclature( total_throughput, total_throughput_string, 13 );
 
-    /* Blank abreviations used in summary table B=blank, NB=no blank */
+    /* Blank abbreviations used in summary table B=blank, NB=no blank */
     if( nwipe_options.noblank )
     {
         strcpy( blank, "NB" );
@@ -878,7 +869,7 @@ void nwipe_log_summary( nwipe_context_t** ptr, int nwipe_selected )
         strcpy( blank, "B" );
     }
 
-    /* Verify abreviations used in summary table */
+    /* Verify abbreviations used in summary table */
     switch( nwipe_options.verify )
     {
         case NWIPE_VERIFY_NONE:
@@ -985,3 +976,32 @@ void convert_seconds_to_hours_minutes_seconds( u64 total_seconds, int* hours, in
         }
     }
 }
+
+int nwipe_strip_path( char* output, char* input )
+{
+    /* Take the input string, say "/dev/sda" and remove the "/dev/", prefix the result
+     * with 'length' spaces. So if length=8 and input=/dev/sda, output will
+     * be "     sda", a string 8 characters long right justified with spaces.
+     */
+    int idx_dest;
+    int idx_src;
+    idx_dest = 8;
+    // idx_dest = length;
+    output[idx_dest--] = 0;
+    idx_src = strlen( input );
+    idx_src--;
+
+    while( idx_dest >= 0 )
+    {
+        /* if the device name contains a / start prefixing spaces */
+        if( input[idx_src] == '/' )
+        {
+            output[idx_dest--] = ' ';
+            continue;
+        }
+        if( idx_src >= 0 )
+        {
+            output[idx_dest--] = input[idx_src--];
+        }
+    }
+}
