@@ -63,8 +63,10 @@ const char* nwipe_dodshort_label = "DoD Short";
 const char* nwipe_gutmann_label = "Gutmann Wipe";
 const char* nwipe_ops2_label = "RCMP TSSIT OPS-II";
 const char* nwipe_random_label = "PRNG Stream";
-const char* nwipe_zero_label = "Zero Fill";
-const char* nwipe_verify_label = "Verify Blank";
+const char* nwipe_zero_label = "Fill With Zeros";
+const char* nwipe_one_label = "Fill With Ones";
+const char* nwipe_verify_zero_label = "Verify Zeros (0x00)";
+const char* nwipe_verify_one_label = "Verify Ones  (0xFF)";
 const char* nwipe_is5enh_label = "HMG IS5 Enhanced";
 
 const char* nwipe_unknown_label = "Unknown Method (FIXME)";
@@ -100,9 +102,17 @@ const char* nwipe_method_label( void* method )
     {
         return nwipe_zero_label;
     }
-    if( method == &nwipe_verify )
+    if( method == &nwipe_one )
     {
-        return nwipe_verify_label;
+        return nwipe_one_label;
+    }
+    if( method == &nwipe_verify_zero )
+    {
+        return nwipe_verify_zero_label;
+    }
+    if( method == &nwipe_verify_one )
+    {
+        return nwipe_verify_one_label;
     }
     if( method == &nwipe_is5enh )
     {
@@ -147,10 +157,73 @@ void* nwipe_zero( void* ptr )
     return NULL;
 } /* nwipe_zero */
 
-void* nwipe_verify( void* ptr )
+void* nwipe_one( void* ptr )
 {
     /**
-     * Fill the device with zeroes.
+     * Fill the device with ones.
+     */
+
+    nwipe_context_t* c;
+    c = (nwipe_context_t*) ptr;
+
+    /* get current time at the start of the wipe  */
+    time( &c->start_time );
+
+    /* set wipe in progress flag for GUI */
+    c->wipe_status = 1;
+
+    /* setup for a zero-fill. */
+
+    char onefill[1] = { '\xFF' };
+    nwipe_pattern_t patterns[] = { { 1, &onefill[0] },  // pass 1: 1s
+                                   { 0, NULL } };
+
+    /* Run the method. */
+    c->result = nwipe_runmethod( c, patterns );
+
+    /* Finished. Set the wipe_status flag so that the GUI knows */
+    c->wipe_status = 0;
+
+    /* get current time at the end of the wipe  */
+    time( &c->end_time );
+
+    return NULL;
+} /* nwipe_one */
+
+void* nwipe_verify_zero( void* ptr )
+{
+    /**
+     * Verify the device is full of zeros.
+     */
+
+    nwipe_context_t* c;
+    c = (nwipe_context_t*) ptr;
+
+    /* get current time at the start of the wipe  */
+    time( &c->start_time );
+
+    /* set wipe in progress flag for GUI */
+    c->wipe_status = 1;
+
+    /* Do nothing because nwipe_runmethod appends a zero-fill. */
+    nwipe_pattern_t patterns[] = { { 0, NULL } };
+
+    /* Run the method. */
+    c->result = nwipe_runmethod( c, patterns );
+
+    /* Finished. Set the wipe_status flag so that the GUI knows */
+    c->wipe_status = 0;
+
+    /* get current time at the end of the wipe  */
+    time( &c->end_time );
+
+    return NULL;
+} /* nwipe_verify zeros */
+
+void* nwipe_verify_one( void* ptr )
+{
+    /**
+     * Verify the device is full of ones.
      */
 
     nwipe_context_t* c;
@@ -335,18 +408,6 @@ void* nwipe_gutmann( void* ptr )
     /* set wipe in progress flag for GUI */
     c->wipe_status = 1;
 
-    /* A result buffer. */
-    int r;
-
-    /* The number of patterns in the Guttman Wipe, also used to index the 'patterns' array. */
-    int i = 35;
-
-    /* An index into the 'book' array. */
-    int j;
-
-    /* The N-th element that has not been used. */
-    int n;
-
     /* Define the Gutmann method. */
     nwipe_pattern_t book[] = { { -1, "" },  // Random pass.
                                { -1, "" },  // Random pass.
@@ -389,11 +450,10 @@ void* nwipe_gutmann( void* ptr )
     nwipe_pattern_t patterns[36];
 
     /* An entropy buffer. */
-    u16 s[i];
+    u16 s[27];
 
     /* Load the array with random characters. */
-    r = read( c->entropy_fd, &s, sizeof( s ) );
-
+    ssize_t r = read( c->entropy_fd, &s, sizeof( s ) );
     if( r != sizeof( s ) )
     {
         r = errno;
@@ -413,14 +473,19 @@ void* nwipe_gutmann( void* ptr )
         }
     }
 
-    while( --i >= 0 )
+    // First 4 random passes
+    for( int i = 0; i <= 3; ++i )
+    {
+        patterns[i] = book[i];
+    }
+    // Middle 27 passes in random order
+    for( int i = 26; i >= 0; --i )
     {
-
         /* Get a random integer that is less than the first index 'i'. */
-        n = (int) ( (double) ( s[i] ) / (double) ( 0x0000FFFF + 1 ) * (double) ( i + 1 ) );
+        int n = (int) ( (double) ( s[i] ) / (double) ( 0x0000FFFF + 1 ) * (double) ( i + 1 ) );
 
         /* Initialize the secondary index. */
-        j = -1;
+        int j = 3;
 
         while( n-- >= 0 )
         {
@@ -435,11 +500,16 @@ void* nwipe_gutmann( void* ptr )
         }
 
         /* Copy the element. */
-        patterns[i] = book[j];
+        patterns[i + 4] = book[j];
 
         /* Mark this element as having been used. */
         book[j].length = 0;
     }
+    // Last 4 random passes
+    for( int i = 31; i <= 34; ++i )
+    {
+        patterns[i] = book[i];
+    }
 
     /* Ensure that the array is terminated. */
     patterns[35].length = 0;
@@ -701,6 +771,9 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
     /* The zero-fill pattern for the final pass of most methods. */
     nwipe_pattern_t pattern_zero = { 1, "\x00" };
 
+    /* The one-fill pattern for verification of the ones fill */
+    nwipe_pattern_t pattern_one = { 1, "\xFF" };
+
     /* Create the PRNG state buffer. */
     c->prng_seed.length = NWIPE_KNOB_PRNG_STATE_LENGTH;
     c->prng_seed.s = malloc( c->prng_seed.length );
@@ -728,10 +801,8 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
     /* For the selected method, calculate the correct round_size value (for correct percentage calculation) */
     calculate_round_size( c );
 
-    c->result = c->round_size;
-
-    /* If only verifing then the round size is the device size */
-    if( nwipe_options.method == &nwipe_verify )
+    /* If only verifying then the round size is the device size */
+    if( nwipe_options.method == &nwipe_verify_zero || nwipe_options.method == &nwipe_verify_one )
     {
         c->round_size = c->device_size;
     }
@@ -994,9 +1065,9 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
 
     } /* final ops2 */
 
-    else if( nwipe_options.method == &nwipe_verify )
+    else if( nwipe_options.method == &nwipe_verify_zero )
     {
-        nwipe_log( NWIPE_LOG_NOTICE, "Verifying that %s is empty", c->device_name );
+        nwipe_log( NWIPE_LOG_NOTICE, "Verifying that %s is zeroed", c->device_name );
 
         /* Verify the final zero pass. */
         c->pass_type = NWIPE_PASS_VERIFY;
@@ -1010,11 +1081,36 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
         }
         if( c->verify_errors == 0 )
         {
-            nwipe_log( NWIPE_LOG_NOTICE, "[SUCCESS] Verified that %s is empty.", c->device_name );
+            nwipe_log( NWIPE_LOG_NOTICE, "[SUCCESS] Verified that %s is Zeroed.", c->device_name );
         }
         else
         {
-            nwipe_log( NWIPE_LOG_ERROR, "[FAILURE] %s is not empty .", c->device_name );
+            nwipe_log( NWIPE_LOG_ERROR, "[FAILURE] %s has not been Zeroed .", c->device_name );
+        }
+
+    } /* verify */
+
+    else if( nwipe_options.method == &nwipe_verify_one )
+    {
+        nwipe_log( NWIPE_LOG_NOTICE, "Verifying that %s is Ones (0xFF)", c->device_name );
+
+        /* Verify the final ones pass. */
+        c->pass_type = NWIPE_PASS_VERIFY;
+        r = nwipe_static_verify( c, &pattern_one );
+        c->pass_type = NWIPE_PASS_NONE;
+
+        /* Check for a fatal error. */
+        if( r < 0 )
+        {
+            return r;
+        }
+        if( c->verify_errors == 0 )
+        {
+            nwipe_log( NWIPE_LOG_NOTICE, "[SUCCESS] Verified that %s is full of ones (0xFF).", c->device_name );
+        }
+        else
+        {
+            nwipe_log( NWIPE_LOG_ERROR, "[FAILURE] %s is not full of ones (0xFF).", c->device_name );
         }
 
     } /* verify */
@@ -1206,7 +1302,7 @@ void calculate_round_size( nwipe_context_t* c )
                 c->round_size += c->device_size;
             }
 
-            /* As no final zero blanking pass is permitted by this standard reduce round size if it's seelected */
+            /* As no final zero blanking pass is permitted by this standard reduce round size if it's selected */
             if( nwipe_options.noblank == 0 )
             {
                 /* Reduce for blanking pass */
