@@ -41,6 +41,7 @@
 #include "device.h"
 #include "logging.h"
 #include "gui.h"
+#include "temperature.h"
 
 #include <sys/ioctl.h> /* FIXME: Twice Included */
 #include <sys/shm.h>
@@ -60,9 +61,17 @@ int main( int argc, char** argv )
     int nwipe_enumerated;  // The number of contexts that have been enumerated.
     int nwipe_error = 0;  // An error counter.
     int nwipe_selected = 0;  // The number of contexts that have been selected.
+    int any_threads_still_running;  // used in wipe thread cancellation wait loop
+    int thread_timeout_counter;  // timeout thread cancellation after THREAD_CANCELLATION_TIMEOUT seconds
     pthread_t nwipe_gui_thread = 0;  // The thread ID of the GUI thread.
     pthread_t nwipe_sigint_thread;  // The thread ID of the sigint handler.
 
+    char modprobe_command[] = "modprobe %s";
+    char modprobe_command2[] = "/sbin/modprobe %s";
+    char modprobe_command3[] = "/usr/sbin/modprobe %s";
+    char module_shortform[50];
+    char final_cmd_modprobe[sizeof( modprobe_command ) + sizeof( module_shortform )];
+
     /* The entropy source file handle. */
     int nwipe_entropy;
 
@@ -91,7 +100,7 @@ int main( int argc, char** argv )
     /* Initialise, flag indicating whether a wipe has actually started or not 0=no, 1=yes */
     global_wipe_status = 0;
 
-    /* Initialise flags that indicates whether a fatal or non fatal error occured on ANY drive */
+    /* Initialise flags that indicate whether a fatal or non fatal error occurred on ANY drive */
     int fatal_errors_flag = 0;
     int non_fatal_errors_flag = 0;
 
@@ -107,11 +116,18 @@ int main( int argc, char** argv )
 
     /* Parse command line options. */
     nwipe_optind = nwipe_options_parse( argc, argv );
+
     if( nwipe_optind == argc )
     {
         /* File names were not given by the user.  Scan for devices. */
         nwipe_enumerated = nwipe_device_scan( &c1 );
 
+        if( terminate_signal == 1 )
+        {
+            cleanup();
+            exit( 1 );
+        }
+
         if( nwipe_enumerated == 0 )
         {
             nwipe_log( NWIPE_LOG_INFO,
@@ -202,6 +218,53 @@ int main( int argc, char** argv )
         pthread_create( &nwipe_sigint_thread, &pthread_attr, signal_hand, &nwipe_thread_data_ptr );
     }
 
+    /* Makesure the drivetemp module is loaded, else drives hwmon entries won't appear in /sys/class/hwmon */
+    final_cmd_modprobe[0] = 0;
+
+    /* The kernel module we are going to load */
+    strcpy( module_shortform, "drivetemp" );
+
+    /* Determine whether we can access modprobe, required if the PATH environment is not setup ! (Debian sid 'su' as
+     * opposed to 'su -' */
+
+    if( system( "which modprobe > /dev/null 2>&1" ) )
+    {
+        if( system( "which /sbin/modprobe > /dev/null 2>&1" ) )
+        {
+            if( system( "which /usr/sbin/modprobe > /dev/null 2>&1" ) )
+            {
+                nwipe_log( NWIPE_LOG_WARNING, "modprobe command not found. Install kmod package (modprobe)) !" );
+                nwipe_log( NWIPE_LOG_WARNING, "Most temperature monitoring may be unavailable as module drivetemp" );
+                nwipe_log( NWIPE_LOG_WARNING, "could not be loaded. drivetemp is not available on kernels < v5.5" );
+            }
+            else
+            {
+                sprintf( final_cmd_modprobe, modprobe_command3, module_shortform );
+            }
+        }
+        else
+        {
+            sprintf( final_cmd_modprobe, modprobe_command2, module_shortform );
+        }
+    }
+    else
+    {
+        sprintf( final_cmd_modprobe, modprobe_command, module_shortform );
+    }
+
+    /* load the drivetemp module */
+    if( system( final_cmd_modprobe ) != 0 )
+    {
+        nwipe_log( NWIPE_LOG_WARNING, "hwmon: Unable to load module drivetemp, temperatures may be unavilable." );
+        nwipe_log( NWIPE_LOG_WARNING, "hwmon: It's possible the drivetemp software isn't modular but built-in" );
+        nwipe_log( NWIPE_LOG_WARNING, "hwmon: to the kernel, as is the case with ShredOS.x86_64 in which case" );
+        nwipe_log( NWIPE_LOG_WARNING, "hwmon: the temperatures will actually be available despite this issue." );
+    }
+    else
+    {
+        nwipe_log( NWIPE_LOG_NOTICE, "hwmon: Module drivetemp loaded, drive temperatures available" );
+    }
+
     /* A context struct for each device has already been created. */
     /* Now set specific nwipe options */
     for( i = 0; i < nwipe_enumerated; i++ )
@@ -223,12 +286,21 @@ int main( int argc, char** argv )
             /* The user must manually select devices. */
             c1[i]->select = NWIPE_SELECT_FALSE;
         }
+
+        /* Initialise temperature variables for device */
+        nwipe_init_temperature( c1[i] );
+        if( nwipe_options.verbose )
+        {
+            nwipe_log( NWIPE_LOG_NOTICE, "hwmon: Device %s hwmon path = %s", c1[i]->device_name, c1[i]->temp1_path );
+        }
+
+        nwipe_update_temperature( c1[i] );
     }
 
     /* Check for initialization errors. */
     if( nwipe_error )
     {
-        nwipe_log( NWIPE_LOG_ERROR, "Initialization eror %i\n", nwipe_error );
+        nwipe_log( NWIPE_LOG_ERROR, "Initialization error %i\n", nwipe_error );
         cleanup();
         return -1;
     }
@@ -271,6 +343,9 @@ int main( int argc, char** argv )
         {
             nwipe_selected += 1;
         }
+
+        /* Initialise the wipe result value */
+        c1[i]->result = 0;
     }
 
     /* Pass the number selected to the struct for other threads */
@@ -422,7 +497,7 @@ int main( int argc, char** argv )
             if( c2[i]->device_size == 0 )
             {
                 nwipe_log( NWIPE_LOG_ERROR,
-                           "%s, sect/blk/dev %llu/%i/%llu",
+                           "%s, sect/blk/dev %i/%i/%llu",
                            c2[i]->device_name,
                            c2[i]->device_sector_size,
                            c2[i]->device_block_size,
@@ -433,7 +508,7 @@ int main( int argc, char** argv )
             else
             {
                 nwipe_log( NWIPE_LOG_NOTICE,
-                           "%s, sect/blk/dev %llu/%i/%llu",
+                           "%s, sect/blk/dev %i/%i/%llu",
                            c2[i]->device_name,
                            c2[i]->device_sector_size,
                            c2[i]->device_block_size,
@@ -521,7 +596,6 @@ int main( int argc, char** argv )
             if( nwipe_options.verbose )
             {
                 nwipe_log( NWIPE_LOG_INFO, "Requesting wipe thread cancellation for %s", c2[i]->device_name );
-                nwipe_log( NWIPE_LOG_INFO, "Please wait.." );
             }
             pthread_cancel( c2[i]->thread );
         }
@@ -543,9 +617,12 @@ int main( int argc, char** argv )
         {
             nwipe_log( NWIPE_LOG_WARNING, "main()>pthread_join():Error when waiting for GUI thread to cancel." );
         }
-        if( nwipe_options.verbose )
+        else
         {
-            nwipe_log( NWIPE_LOG_INFO, "GUI compute_stats thread has been cancelled" );
+            if( nwipe_options.verbose )
+            {
+                nwipe_log( NWIPE_LOG_INFO, "GUI compute_stats thread has been cancelled" );
+            }
         }
     }
 
@@ -556,26 +633,86 @@ int main( int argc, char** argv )
     }
 
     /* Now join the wipe threads and wait until they have terminated */
-    for( i = 0; i < nwipe_selected; i++ )
+    any_threads_still_running = 1;
+    thread_timeout_counter = THREAD_CANCELLATION_TIMEOUT;
+    while( any_threads_still_running )
     {
-
-        if( c2[i]->thread )
+        /* quit waiting if we've tried 'thread_timeout_counter' times */
+        if( thread_timeout_counter == 0 )
         {
-            /* Joins the thread and waits for completion before continuing */
-            r = pthread_join( c2[i]->thread, NULL );
-            if( r != 0 )
-            {
-                nwipe_log( NWIPE_LOG_WARNING, "main()>pthread_join():Error when waiting for wipe thread to cancel." );
-            }
-            c2[i]->thread = 0; /* Zero the thread so we know it's been cancelled */
+            break;
+        }
 
-            if( nwipe_options.verbose )
+        any_threads_still_running = 0;
+        for( i = 0; i < nwipe_selected; i++ )
+        {
+            if( c2[i]->thread )
             {
-                nwipe_log( NWIPE_LOG_INFO, "Wipe thread for device %s has been cancelled", c2[i]->device_name );
-            }
+                printf( "\nWaiting for wipe thread to cancel for %s\n", c2[i]->device_name );
 
-            /* Close the device file descriptor. */
-            close( c2[i]->device_fd );
+                /* Joins the thread and waits for completion before continuing */
+                r = pthread_join( c2[i]->thread, NULL );
+                if( r != 0 )
+                {
+                    nwipe_log( NWIPE_LOG_ERROR,
+                               "Error joining the wipe thread when waiting for thread to cancel.",
+                               c2[i]->device_name );
+
+                    if( r == EDEADLK )
+                    {
+                        nwipe_log( NWIPE_LOG_ERROR,
+                                   "Error joining the wipe thread: EDEADLK: Deadlock detected.",
+                                   c2[i]->device_name );
+                    }
+                    else
+                    {
+                        if( r == EINVAL )
+                        {
+                            nwipe_log( NWIPE_LOG_ERROR,
+                                       "Error joining the wipe thread: %s EINVAL: thread is not joinable.",
+                                       c2[i]->device_name );
+                        }
+                        else
+                        {
+                            if( r == ESRCH )
+                            {
+                                nwipe_log( NWIPE_LOG_ERROR,
+                                           "Error joining the wipe thread: %s ESRCH: no matching thread found",
+                                           c2[i]->device_name );
+                            }
+                        }
+                    }
+
+                    any_threads_still_running = 1;
+                }
+                else
+                {
+                    c2[i]->thread = 0; /* Zero the thread so we know it's been cancelled */
+
+                    if( nwipe_options.verbose )
+                    {
+                        nwipe_log( NWIPE_LOG_INFO, "Wipe thread for device %s has terminated", c2[i]->device_name );
+                    }
+
+                    /* Close the device file descriptor. */
+                    close( c2[i]->device_fd );
+                }
+            }
+        }
+        thread_timeout_counter--;
+        sleep( 1 );
+    }
+    if( nwipe_options.verbose )
+    {
+        for( i = 0; i < nwipe_selected; i++ )
+        {
+            nwipe_log( NWIPE_LOG_DEBUG,
+                       "Status: %s, result=%d, pass_errors=%llu, verify_errors=%llu, fsync_errors=%llu",
+                       c2[i]->device_name,
+                       c2[i]->result,
+                       c2[i]->pass_errors,
+                       c2[i]->verify_errors,
+                       c2[i]->fsyncdata_errors );
         }
     }
 
@@ -592,25 +729,35 @@ int main( int argc, char** argv )
     {
         for( i = 0; i < nwipe_selected; i++ )
         {
-            /* Check for non-fatal errors. */
-            if( c2[i]->result > 0 )
+            /* Check for errors. */
+            if( c2[i]->result != 0 || c2[i]->pass_errors != 0 || c2[i]->verify_errors != 0
+                || c2[i]->fsyncdata_errors != 0 )
             {
-                nwipe_log( NWIPE_LOG_FATAL, "Nwipe exited with non fatal errors on device = %s\n", c2[i]->device_name );
+                /* If the run_method ever returns anything other than zero then makesure there is at least one pass
+                 * error This is so that the log summary tables correctly show a failure when one occurs as it only
+                 * shows pass, verification and fdatasync errors. */
+                if( c2[i]->result != 0 )
+                {
+                    if( c2[i]->pass_errors == 0 )
+                    {
+                        c2[i]->pass_errors = 1;
+                    }
+                }
+
+                nwipe_log( NWIPE_LOG_FATAL,
+                           "Nwipe exited with errors on device = %s, see log for specific error\n",
+                           c2[i]->device_name );
+                nwipe_log( NWIPE_LOG_DEBUG,
+                           "Status: %s, result=%d, pass_errors=%llu, verify_errors=%llu, fsync_errors=%llu",
+                           c2[i]->device_name,
+                           c2[i]->result,
+                           c2[i]->pass_errors,
+                           c2[i]->verify_errors,
+                           c2[i]->fsyncdata_errors );
                 non_fatal_errors_flag = 1;
                 return_status = 1;
             }
         }
-
-        for( i = 0; i < nwipe_selected; i++ )
-        {
-            /* Check for fatal errors. */
-            if( c2[i]->result < 0 )
-            {
-                nwipe_log( NWIPE_LOG_ERROR, "Nwipe exited with fatal errors on device = %s\n", c2[i]->device_name );
-                fatal_errors_flag = 1;
-                return_status = -1;
-            }
-        }
     }
 
     /* Generate and send the drive status summary to the log */
