@@ -40,6 +40,7 @@ int nwipe_options_parse( int argc, char** argv )
 
     extern nwipe_prng_t nwipe_twister;
     extern nwipe_prng_t nwipe_isaac;
+    extern nwipe_prng_t nwipe_isaac64;
 
     /* The getopt() result holder. */
     int nwipe_opt;
@@ -53,7 +54,7 @@ int nwipe_options_parse( int argc, char** argv )
     int i;
 
     /* The list of acceptable short options. */
-    char nwipe_options_short[] = "Vvhl:m:p:r:e:";
+    char nwipe_options_short[] = "Vvhl:m:p:qr:e:";
 
     /* The list of acceptable long options. */
     static struct option nwipe_options_long[] = {
@@ -93,10 +94,13 @@ int nwipe_options_parse( int argc, char** argv )
         /* Whether to allow signals to interrupt a wipe. */
         { "nosignals", no_argument, 0, 0 },
 
-        /* Whether to exit after wiping or wait for a keypress. */
+        /* Whether to display the gui. */
         { "nogui", no_argument, 0, 0 },
 
-        /* A flag to indicate whether the devices whould be opened in sync mode. */
+        /* Whether to anonymize the serial numbers. */
+        { "quiet", no_argument, 0, 'q' },
+
+        /* A flag to indicate whether the devices would be opened in sync mode. */
         { "sync", required_argument, 0, 0 },
 
         /* Verify that wipe patterns are being written to the device. */
@@ -115,13 +119,14 @@ int nwipe_options_parse( int argc, char** argv )
     nwipe_options.autonuke = 0;
     nwipe_options.autopoweroff = 0;
     nwipe_options.method = &nwipe_dodshort;
-    nwipe_options.prng = &nwipe_twister;
+    nwipe_options.prng = ( sizeof( unsigned long int ) >= 8 ) ? &nwipe_isaac64 : &nwipe_isaac;
     nwipe_options.rounds = 1;
     nwipe_options.noblank = 0;
     nwipe_options.nousb = 0;
     nwipe_options.nowait = 0;
     nwipe_options.nosignals = 0;
     nwipe_options.nogui = 0;
+    nwipe_options.quiet = 0;
     nwipe_options.sync = DEFAULT_SYNC_RATE;
     nwipe_options.verbose = 0;
     nwipe_options.verify = NWIPE_VERIFY_LAST;
@@ -276,9 +281,21 @@ int nwipe_options_parse( int argc, char** argv )
                     break;
                 }
 
-                if( strcmp( optarg, "verify" ) == 0 )
+                if( strcmp( optarg, "one" ) == 0 )
                 {
-                    nwipe_options.method = &nwipe_verify;
+                    nwipe_options.method = &nwipe_one;
+                    break;
+                }
+
+                if( strcmp( optarg, "verify_zero" ) == 0 )
+                {
+                    nwipe_options.method = &nwipe_verify_zero;
+                    break;
+                }
+
+                if( strcmp( optarg, "verify_one" ) == 0 )
+                {
+                    nwipe_options.method = &nwipe_verify_one;
                     break;
                 }
 
@@ -359,10 +376,21 @@ int nwipe_options_parse( int argc, char** argv )
                     break;
                 }
 
+                if( strcmp( optarg, "isaac64" ) == 0 )
+                {
+                    nwipe_options.prng = &nwipe_isaac64;
+                    break;
+                }
+
                 /* Else we do not know this PRNG. */
                 fprintf( stderr, "Error: Unknown prng '%s'.\n", optarg );
                 exit( EINVAL );
 
+            case 'q': /* Anonymize serial numbers */
+
+                nwipe_options.quiet = 1;
+                break;
+
             case 'r': /* Rounds option. */
 
                 if( sscanf( optarg, " %i", &nwipe_options.rounds ) != 1 || nwipe_options.rounds < 1 )
@@ -401,6 +429,7 @@ void nwipe_options_log( void )
 {
     extern nwipe_prng_t nwipe_twister;
     extern nwipe_prng_t nwipe_isaac;
+    extern nwipe_prng_t nwipe_isaac64;
 
     /**
      *  Prints a manifest of options to the log.
@@ -460,11 +489,19 @@ void nwipe_options_log( void )
         }
         else
         {
-            nwipe_log( NWIPE_LOG_NOTICE, "  prng     = Undefined" );
+            if( nwipe_options.prng == &nwipe_isaac64 )
+            {
+                nwipe_log( NWIPE_LOG_NOTICE, "  prng     = Isaac64" );
+            }
+            else
+            {
+                nwipe_log( NWIPE_LOG_NOTICE, "  prng     = Undefined" );
+            }
         }
     }
 
     nwipe_log( NWIPE_LOG_NOTICE, "  method   = %s", nwipe_method_label( nwipe_options.method ) );
+    nwipe_log( NWIPE_LOG_NOTICE, "  quiet    = %i", nwipe_options.quiet );
     nwipe_log( NWIPE_LOG_NOTICE, "  rounds   = %i", nwipe_options.rounds );
     nwipe_log( NWIPE_LOG_NOTICE, "  sync     = %i", nwipe_options.sync );
 
@@ -529,9 +566,14 @@ void display_help()
     puts( "                          ops2                   - RCMP TSSIT OPS-II" );
     puts( "                          random / prng / stream - PRNG Stream" );
     puts( "                          zero / quick           - Overwrite with zeros" );
-    puts( "                          verify                 - Verifies disk is zero filled\n" );
+    puts( "                          one                    - Overwrite with ones (0xFF)" );
+    puts( "                          verify_zero            - Verifies disk is zero filled" );
+    puts( "                          verify_one             - Verifies disk is 0xFF filled\n" );
     puts( "  -l, --logfile=FILE      Filename to log to. Default is STDOUT\n" );
-    puts( "  -p, --prng=METHOD       PRNG option (mersenne|twister|isaac)\n" );
+    puts( "  -p, --prng=METHOD       PRNG option (mersenne|twister|isaac|isaac64)\n" );
+    puts( "  -q, --quiet             Anonymize logs and the GUI by removing unique data, i.e." );
+    puts( "                          serial numbers, LU WWN Device ID, and SMBIOS/DMI data" );
+    puts( "                          XXXXXX = S/N exists, ????? = S/N not obtainable\n" );
     puts( "  -r, --rounds=NUM        Number of times to wipe the device using the selected" );
     puts( "                          method (default: 1)\n" );
     puts( "      --noblank           Do NOT blank disk after wipe" );
